// Copyright (c) 2019-2020 The Authors
// This file is part of @npcz/ngx-highlight
// https://github.com/abdes/happy-coding
//
// SPDX-License-Identifier: BSD-3-Clause
import { __assign, __decorate, __metadata } from "tslib";
import { Directive, ElementRef, HostListener, Input, Renderer2, OnDestroy, } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
var HighlightDirective = /** @class */ (function () {
    function HighlightDirective(_renderer, _el) {
        this._renderer = _renderer;
        this._el = _el;
        this.DEFAULT_CONFIG = {
            hover: 'none',
            focus: 'none',
            debounceTime: 50,
        };
        // Automatic unsubscription when the component is destroyed
        this._style = __assign({}, this.DEFAULT_CONFIG);
        this._hovered$ = new BehaviorSubject(false);
        this._isHovered = false;
        this._focused$ = new BehaviorSubject(false);
        this._isFocused = false;
        this._subscriptions = {
            hover: undefined,
            focus: undefined,
        };
    }
    Object.defineProperty(HighlightDirective.prototype, "highlightStyle", {
        get: function () {
            return this._style;
        },
        set: function (value) {
            this._updateStyleConfig(value);
        },
        enumerable: true,
        configurable: true
    });
    HighlightDirective.prototype.ngOnDestroy = function () {
        this._stopFocusSubscription();
        this._startHoverSubscription();
    };
    HighlightDirective.prototype.onMouseEnter = function () {
        var _a;
        (_a = this._hovered$) === null || _a === void 0 ? void 0 : _a.next(true);
    };
    HighlightDirective.prototype.onMouseLeave = function () {
        var _a;
        (_a = this._hovered$) === null || _a === void 0 ? void 0 : _a.next(false);
    };
    HighlightDirective.prototype.onFocusIn = function () {
        var _a;
        (_a = this._focused$) === null || _a === void 0 ? void 0 : _a.next(true);
    };
    HighlightDirective.prototype.onFocusOut = function () {
        var _a;
        (_a = this._focused$) === null || _a === void 0 ? void 0 : _a.next(false);
    };
    HighlightDirective.prototype._updateStyleConfig = function (value) {
        // Adjust debounceTime if it is negative
        var adjustedDebounceTime = this.DEFAULT_CONFIG.debounceTime;
        if (value === null || value === void 0 ? void 0 : value.debounceTime) {
            adjustedDebounceTime = value.debounceTime > 0 ? value.debounceTime : 0;
        }
        // When the debounceTime changes, always stop the subscription
        // to restart it again
        if (this._style.debounceTime != adjustedDebounceTime) {
            this._stopFocusSubscription();
            this._stopHoverSubscription();
        }
        // Merge the provided config with the default config
        value &&
            (this._style = __assign(__assign(__assign({}, this.DEFAULT_CONFIG), value), { debounceTime: adjustedDebounceTime }));
        // Hover: Start observing if we're not already doing that
        if (this._style.hover != 'none' && !this._subscriptions.hover) {
            this._startHoverSubscription();
        }
        if (this._style.hover == 'none' && this._subscriptions.hover) {
            this._stopHoverSubscription();
        }
        // Focus: Start observing if we're not already doing that
        if (this._style.focus != 'none' && !this._subscriptions.focus) {
            this._startFocusSubscription();
        }
        if (this._style.focus == 'none' && this._subscriptions.focus) {
            this._stopFocusSubscription();
        }
    };
    HighlightDirective.prototype._startHoverSubscription = function () {
        var _this = this;
        this._subscriptions.hover = this._hovered$
            .pipe(debounceTime(this._style.debounceTime))
            .subscribe(function (status) {
            _this._isHovered = status;
            // Do not apply the hover effect if the element is focused
            if (_this._isHovered && !_this._isFocused)
                _this._applyEffect('hover');
            else
                _this._removeEffect('hover');
        });
        this._hovered$.next(this._isHovered);
    };
    HighlightDirective.prototype._stopHoverSubscription = function () {
        var _a;
        (_a = this._subscriptions.hover) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._subscriptions.hover = undefined;
    };
    HighlightDirective.prototype._startFocusSubscription = function () {
        var _this = this;
        this._subscriptions.focus = this._focused$
            .pipe(debounceTime(this._style.debounceTime))
            .subscribe(function (status) {
            _this._isFocused = status;
            if (_this._isFocused) {
                _this._applyEffect('focus');
                // If the element is hovered, then remove the hover effect
                if (_this._isHovered)
                    _this._removeEffect('hover');
            }
            else {
                _this._removeEffect('focus');
                // If the element is hovered, then apply the hover effect
                if (_this._isHovered)
                    _this._applyEffect('hover');
            }
        });
        this._focused$.next(this._isFocused);
    };
    HighlightDirective.prototype._stopFocusSubscription = function () {
        var _a;
        (_a = this._subscriptions.focus) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._subscriptions.focus = undefined;
    };
    HighlightDirective.prototype._applyEffect = function (effect) {
        switch (this._style[effect]) {
            case 'shadow':
                this._renderer.addClass(this._el.nativeElement, "hc-" + effect + "-shadow");
                break;
            case 'outline':
                this._renderer.addClass(this._el.nativeElement, "hc-" + effect + "-outline");
                break;
            case 'background':
                this._renderer.addClass(this._el.nativeElement, "hc-" + effect + "-background");
                break;
            // Default is 'none'
            default:
        }
    };
    HighlightDirective.prototype._removeEffect = function (effect) {
        switch (this._style[effect]) {
            case 'shadow':
                this._renderer.removeClass(this._el.nativeElement, "hc-" + effect + "-shadow");
                break;
            case 'outline':
                this._renderer.removeClass(this._el.nativeElement, "hc-" + effect + "-outline");
                break;
            case 'background':
                this._renderer.removeClass(this._el.nativeElement, "hc-" + effect + "-background");
                break;
            // Default is 'none'
            default:
        }
    };
    HighlightDirective.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef }
    ]; };
    __decorate([
        Input('hcHighlight'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], HighlightDirective.prototype, "highlightStyle", null);
    __decorate([
        HostListener('mouseenter'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onMouseEnter", null);
    __decorate([
        HostListener('mouseleave'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onMouseLeave", null);
    __decorate([
        HostListener('focusin'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onFocusIn", null);
    __decorate([
        HostListener('focusout'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], HighlightDirective.prototype, "onFocusOut", null);
    HighlightDirective = __decorate([
        Directive({
            selector: '[hcHighlight]',
        }),
        __metadata("design:paramtypes", [Renderer2, ElementRef])
    ], HighlightDirective);
    return HighlightDirective;
}());
export { HighlightDirective };
//# sourceMappingURL=highlight.directive.js.map