import { __decorate, __metadata } from 'tslib';
import { Renderer2, ElementRef, Input, HostListener, Directive, NgModule } from '@angular/core';
import { BehaviorSubject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

// Copyright (c) 2019-2020 The Authors
let HighlightDirective = class HighlightDirective {
    constructor(_renderer, _el) {
        this._renderer = _renderer;
        this._el = _el;
        this.DEFAULT_CONFIG = {
            hover: 'none',
            focus: 'none',
            debounceTime: 50,
        };
        // Automatic unsubscription when the component is destroyed
        this._style = Object.assign({}, this.DEFAULT_CONFIG);
        this._hovered$ = new BehaviorSubject(false);
        this._isHovered = false;
        this._focused$ = new BehaviorSubject(false);
        this._isFocused = false;
        this._subscriptions = {
            hover: undefined,
            focus: undefined,
        };
    }
    get highlightStyle() {
        return this._style;
    }
    set highlightStyle(value) {
        this._updateStyleConfig(value);
    }
    ngOnDestroy() {
        this._stopFocusSubscription();
        this._startHoverSubscription();
    }
    onMouseEnter() {
        var _a;
        (_a = this._hovered$) === null || _a === void 0 ? void 0 : _a.next(true);
    }
    onMouseLeave() {
        var _a;
        (_a = this._hovered$) === null || _a === void 0 ? void 0 : _a.next(false);
    }
    onFocusIn() {
        var _a;
        (_a = this._focused$) === null || _a === void 0 ? void 0 : _a.next(true);
    }
    onFocusOut() {
        var _a;
        (_a = this._focused$) === null || _a === void 0 ? void 0 : _a.next(false);
    }
    _updateStyleConfig(value) {
        // Adjust debounceTime if it is negative
        let adjustedDebounceTime = this.DEFAULT_CONFIG.debounceTime;
        if (value === null || value === void 0 ? void 0 : value.debounceTime) {
            adjustedDebounceTime = value.debounceTime > 0 ? value.debounceTime : 0;
        }
        // When the debounceTime changes, always stop the subscription
        // to restart it again
        if (this._style.debounceTime != adjustedDebounceTime) {
            this._stopFocusSubscription();
            this._stopHoverSubscription();
        }
        // Merge the provided config with the default config
        value &&
            (this._style = Object.assign(Object.assign(Object.assign({}, this.DEFAULT_CONFIG), value), { debounceTime: adjustedDebounceTime }));
        // Hover: Start observing if we're not already doing that
        if (this._style.hover != 'none' && !this._subscriptions.hover) {
            this._startHoverSubscription();
        }
        if (this._style.hover == 'none' && this._subscriptions.hover) {
            this._stopHoverSubscription();
        }
        // Focus: Start observing if we're not already doing that
        if (this._style.focus != 'none' && !this._subscriptions.focus) {
            this._startFocusSubscription();
        }
        if (this._style.focus == 'none' && this._subscriptions.focus) {
            this._stopFocusSubscription();
        }
    }
    _startHoverSubscription() {
        this._subscriptions.hover = this._hovered$
            .pipe(debounceTime(this._style.debounceTime))
            .subscribe((status) => {
            this._isHovered = status;
            // Do not apply the hover effect if the element is focused
            if (this._isHovered && !this._isFocused)
                this._applyEffect('hover');
            else
                this._removeEffect('hover');
        });
        this._hovered$.next(this._isHovered);
    }
    _stopHoverSubscription() {
        var _a;
        (_a = this._subscriptions.hover) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._subscriptions.hover = undefined;
    }
    _startFocusSubscription() {
        this._subscriptions.focus = this._focused$
            .pipe(debounceTime(this._style.debounceTime))
            .subscribe((status) => {
            this._isFocused = status;
            if (this._isFocused) {
                this._applyEffect('focus');
                // If the element is hovered, then remove the hover effect
                if (this._isHovered)
                    this._removeEffect('hover');
            }
            else {
                this._removeEffect('focus');
                // If the element is hovered, then apply the hover effect
                if (this._isHovered)
                    this._applyEffect('hover');
            }
        });
        this._focused$.next(this._isFocused);
    }
    _stopFocusSubscription() {
        var _a;
        (_a = this._subscriptions.focus) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this._subscriptions.focus = undefined;
    }
    _applyEffect(effect) {
        switch (this._style[effect]) {
            case 'shadow':
                this._renderer.addClass(this._el.nativeElement, `hc-${effect}-shadow`);
                break;
            case 'outline':
                this._renderer.addClass(this._el.nativeElement, `hc-${effect}-outline`);
                break;
            case 'background':
                this._renderer.addClass(this._el.nativeElement, `hc-${effect}-background`);
                break;
            // Default is 'none'
            default:
        }
    }
    _removeEffect(effect) {
        switch (this._style[effect]) {
            case 'shadow':
                this._renderer.removeClass(this._el.nativeElement, `hc-${effect}-shadow`);
                break;
            case 'outline':
                this._renderer.removeClass(this._el.nativeElement, `hc-${effect}-outline`);
                break;
            case 'background':
                this._renderer.removeClass(this._el.nativeElement, `hc-${effect}-background`);
                break;
            // Default is 'none'
            default:
        }
    }
};
HighlightDirective.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef }
];
__decorate([
    Input('hcHighlight'),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], HighlightDirective.prototype, "highlightStyle", null);
__decorate([
    HostListener('mouseenter'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], HighlightDirective.prototype, "onMouseEnter", null);
__decorate([
    HostListener('mouseleave'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], HighlightDirective.prototype, "onMouseLeave", null);
__decorate([
    HostListener('focusin'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], HighlightDirective.prototype, "onFocusIn", null);
__decorate([
    HostListener('focusout'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], HighlightDirective.prototype, "onFocusOut", null);
HighlightDirective = __decorate([
    Directive({
        selector: '[hcHighlight]',
    }),
    __metadata("design:paramtypes", [Renderer2, ElementRef])
], HighlightDirective);

// Copyright (c) 2019-2020 The Authors
let HighlightModule = class HighlightModule {
};
HighlightModule = __decorate([
    NgModule({
        declarations: [HighlightDirective],
        imports: [CommonModule],
        exports: [HighlightDirective],
    })
], HighlightModule);

// Copyright (c) 2019-2020 The Authors

// Copyright (c) 2019-2020 The Authors

/**
 * Generated bundle index. Do not edit.
 */

export { HighlightDirective, HighlightModule };
//# sourceMappingURL=npcz-ngx-highlight.js.map
